//
//  MIDIDriver.h
//  Obie
//
//  Created by groumpf on Sat Apr 03 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreMIDI/MIDIServices.h>

// sound
#define PATCH_TAB_SIZE 384
// 384 parametres + checksum + sysex
#define PATCH_SYSEX_SIZE  (PATCH_TAB_SIZE+1+7)
#define SOUND_PARAMETER_CHANGE_TYPE 0x20
#define SOUND_DUMP_TYPE 0x10

// multi
#define MULTI_TAB_SIZE 385
#define MULTI_SYSEX_SIZE (MULTI_TAB_SIZE+8)
#define MULTI_PARAMETER_CHANGE_TYPE 0x21
#define MULTI_DUMP_TYPE 0x11

// drum map
#define DRUMMAP_TAB_SIZE 393
#define DRUMMAP_SYSEX_SIZE (DRUMMAP_TAB_SIZE+8)
#define DRUMMAP_PARAMETER_CHANGE_TYPE 0x22
#define DRUMMAP_DUMP_TYPE 0x12

#define SOUND_SEND_TO_EDITBUFFER 0x20
#define SOUND_SEND_TO_MULTI_INST 0x30
#define MULTI_SEND_TO_EDITBUFFER 0x20


@interface MIDIDriver : NSObject {
	MIDIClientRef mMIDIclient;
	MIDIPortRef 	mInPort;
	MIDIPortRef		mOutPort;
	MIDIEndpointRef	mDest;
	MIDIEndpointRef mSource;
	int mInputPortNumber;
	int mOutputPortNumber;
}

// singleton
+ (MIDIDriver *)sharedInstance;

- (void)setMIDIInput:(int)aInputPort Output:(int)aOutputPort;
- (NSArray *)midiInputs;
- (NSArray *)midiOutputs;
-(int)inputPortNumber;
-(int)outputPortNumber;

- (void)reset;


// envoi d'un parameter seul
- (void)sendParameter: (int)aParameter Value:(int) aValue type:(int)aParameterChangeType part:(int)aPart;

// demande de dump du sound courant
- (OSStatus)sendRequestSound;

// demande de dump du sound n du multi courant
- (OSStatus)sendRequestMultiSound:(int)aSoundNumber;

- (OSStatus)sendRequestSound:(int)aType number:(int)aNumber;

// demande de dump du multi courant
- (OSStatus)sendRequestMulti;

// envoi le patch dans le buffer d'edition
// aDumpType type de dump sound ou multi
- (void)sendPatch:(uint8_t*)aPatch type:(int)aDumpType;

- (void)storePatch:(uint8_t*)aPatch Bank:(int)aBank Number:(int)aNumber type:(int)aDumpType;

// aType : 0x20 pour le buffer d'edition
//         0x30 pour le buffer d'un instrument multi
//         0x40 pour la banque A
//         0x41 pour la banque B
//         0x42 pour la banque C
// aNumber: numero du patch 0-0x63
-(void)sendPatch:(uint8_t*)aPatch Type:(int)aType Number:(int)aNumber dumpType:(int)aDumpType;

//
// recupere les donnees recues apres un appel a sendRequestDataType
- (int)getReceivedBytes:(uint8_t *)aPatch maxSize:(int)aMaxSize;

@end
